import React, {
  useState,
  useEffect,
  useCallback,
  useContext,
  useMemo
} from "react";
import {
  Link,
  Container,
  IconButton,
  Card,
  CardContent,
  Typography
} from "@mui/material";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import { useHostIOCList } from "../../api/SwaggerApi";
import { HostBadge } from "../../components/host/HostBadge";
import { SimpleAccordion } from "../../components/common/Accordion/SimpleAccordion";
import { IOCAsyncList } from "../../components/IOC/IOCAsyncList";
import { KeyValueTable } from "@ess-ics/ce-ui-common/dist/components/common/KeyValueTable";
import { LokiPanel } from "../../components/common/Loki/LokiPanel";
import { useNavigate } from "react-router-dom";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import AccessControl from "../../components/auth/AccessControl";
import { useHost } from "../../api/SwaggerApi";
import AlertMessages from "../../components/IOC/AlertMessages";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";
import { GlobalAppBarContext } from "@ess-ics/ce-ui-common";
import { usePagination } from "../../hooks/pagination";
import { RootPaper } from "@ess-ics/ce-ui-common/dist/components/common/container/RootPaper";

// const PREFIX = "HostDetailsView";

// const classes = {
//   secondItem: `${PREFIX}-secondItem`,
//   hostHeader: `${PREFIX}-hostHeader`
// };

// const StyledPaper = styled(Paper)(({ theme }) => ({
//   [`& .${classes.secondItem}`]: {
//     marginTop: theme.spacing(2)
//   },

//   [`& .${classes.hostHeader}`]: {
//     marginTop: theme.spacing(2)
//   }
// }));

export function HostDetailsView({ id }) {
  const [host] = useHost(id);
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => {
    if (host && host.csEntryHost) {
      setTitle(applicationTitle("Host Details: " + host.csEntryHost.name));
    }
  }, [host, setTitle]);

  const [iocs, getIocs /* reset*/, , loading] = useHostIOCList();
  const [deployedIocs, setDeployedIocs] = useState([]);
  // Sync manipulatable ioc list copy
  useEffect(() => {
    setDeployedIocs(iocs?.deployedList ?? []);
  }, [iocs, setDeployedIocs]);

  const [urlState, setUrlState] = useUrlState(
    {
      iocs_rows: "20",
      iocs_page: "0",
      iocs_open: "true",
      syslog_open: "true",
      details_open: "false"
    },
    { navigateMode: "replace" }
  );

  const navigate = useNavigate();

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.iocs_rows),
      page: deserialize(urlState.iocs_page)
    };
  }, [urlState]);

  const setUrlPagination = useCallback(
    ({ rows, page }) => {
      setUrlState({
        iocs_rows: serialize(rows),
        iocs_page: serialize(page)
      });
    },
    [setUrlState]
  );

  const rowsPerPage = [20, 50];

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: urlPagination.iocs_rows ?? rowsPerPage[0],
    initPage: urlPagination.iocs_page ?? 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: iocs.totalCount ?? 0 });
  }, [setPagination, iocs.totalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
  };

  useEffect(() => {
    let requestParams = initRequestParams(urlPagination, null);

    requestParams.host_csentry_id = id;

    getIocs(requestParams);
  }, [getIocs, urlPagination, id]);

  function listToString(list) {
    if (list) {
      if (Array.isArray(list)) {
        return list.join(", ");
      }
      return list;
    }

    return "";
  }

  let renderHost = {
    "registered by": host?.csEntryHost?.user,
    "device type": host?.csEntryHost?.device_type,
    description: host?.csEntryHost?.description,
    scope: host?.csEntryHost?.scope,
    "vm owner": listToString(host?.csEntryHost?.ansible_vars?.vm_owner),
    "ansible groups": listToString(host?.csEntryHost?.ansible_groups)
  };

  const handleClick = (event) => {
    navigate(-1);
  };

  return (
    <RootPaper>
      <IconButton
        color="inherit"
        onClick={handleClick}
        size="large"
      >
        <ArrowBackIcon />
      </IconButton>

      {host && <AlertMessages alerts={host.alerts} />}
      <Card
        variant="outlined"
        sx={{ marginTop: 2 }}
      >
        <CardContent>{host && <HostBadge host={host} />}</CardContent>
      </Card>
      <SimpleAccordion
        summary="Deployed IOCs"
        expanded={deserialize(urlState.iocs_open)}
        onChange={(event, expanded) =>
          setUrlState({ iocs_open: serialize(expanded) })
        }
      >
        <IOCAsyncList
          iocs={deployedIocs}
          setIocs={setDeployedIocs}
          loading={loading}
          rowType="host"
          pagination={pagination}
          onPage={onPage}
        />
      </SimpleAccordion>
      <SimpleAccordion
        summary="Host details"
        expanded={deserialize(urlState.details_open)}
        onChange={(event, expanded) =>
          setUrlState({ details_open: serialize(expanded) })
        }
      >
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          {host && (
            <KeyValueTable
              obj={renderHost}
              variant="overline"
              sx={{ border: 0 }}
              valueOptions={{ headerName: "" }}
            />
          )}
        </AccessControl>
      </SimpleAccordion>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <SimpleAccordion
          summary="Syslog info"
          expanded={deserialize(urlState.syslog_open)}
          onChange={(event, expanded) =>
            setUrlState({ syslog_open: serialize(expanded) })
          }
        >
          {host && (
            <Container>
              <LokiPanel
                host={host}
                isSyslog
                isDeployed
              />
            </Container>
          )}
        </SimpleAccordion>
      </AccessControl>
      <Card variant="outlined">
        <CardContent>
          <Container>
            <Typography variant="h6">Open in CSEntry</Typography>
            <Link
              href={`https://csentry.esss.lu.se/network/hosts/view/${host?.csEntryHost.name}`}
              target="_blank"
              rel="noreferrer"
              underline="hover"
            >{`https://csentry.esss.lu.se/network/hosts/view/${host?.csEntryHost?.name}`}</Link>

            <Typography
              variant="h6"
              sx={{ marginTop: 2 }}
            >
              Open in Grafana
            </Typography>
            <Link
              href={`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.csEntryHost.fqdn}`}
              target="_blank"
              rel="noreferrer"
              underline="hover"
            >{`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.csEntryHost.fqdn}`}</Link>
          </Container>
        </CardContent>
      </Card>
    </RootPaper>
  );
}
