import React, {
  useState,
  useMemo,
  useCallback,
  useContext,
  useEffect
} from "react";
import { useRecordSearch } from "../../api/SwaggerApi";
import {
  Container,
  Grid,
  useMediaQuery,
  Tabs,
  Tab,
  Typography
} from "@mui/material";
import { GlobalAppBarContext, RootPaper } from "@ess-ics/ce-ui-common";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";
import { RecordTable } from "../../components/records/RecordTable";
import { RecordList } from "../../components/records/RecordList";
import { usePagination } from "../../hooks/pagination";

export function RecordListView() {
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => setTitle(applicationTitle("Records")), [setTitle]);

  const [records, getRecords /* reset*/, , loading] = useRecordSearch();

  const [urlState, setUrlState] = useUrlState(
    {
      tab: "0",
      rows: "20",
      page: "0",
      query: ""
    },
    { navigateMode: "replace" }
  );
  const [recordFilter, setRecordFilter] = useState();

  const handleTabChange = useCallback(
    (event, tab) => {
      setUrlState((s) =>
        serialize(s.tab) === serialize(tab)
          ? { tab: serialize(tab) }
          : { tab: serialize(tab), page: "0" }
      );

      changeTab(tab);
    },
    [setUrlState]
  );

  const changeTab = (tab) => {
    if (tab === 0) {
      setRecordFilter(null);
    } else if (tab === 1) {
      setRecordFilter("ACTIVE");
    } else if (tab === 2) {
      setRecordFilter("INACTIVE");
    }
  };

  useEffect(() => {
    urlState.tab && changeTab(deserialize(urlState.tab));
  }, [urlState]);

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.rows),
      page: deserialize(urlState.page)
    };
  }, [urlState.rows, urlState.page]);

  const setUrlPagination = useCallback(
    ({ rows, page }) => {
      setUrlState({
        rows: serialize(rows),
        page: serialize(page)
      });
    },
    [setUrlState]
  );

  const rowsPerPage = [20, 50];

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: urlPagination.rows ?? rowsPerPage[0],
    initPage: urlPagination.page ?? 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: records.totalCount ?? 0 });
  }, [setPagination, records.totalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  // Request new search results whenever search or pagination changes
  useEffect(() => {
    let requestParams = initRequestParams(
      pagination,
      deserialize(urlState.query)
    );
    requestParams.filter = recordFilter;
    getRecords(requestParams);
  }, [getRecords, recordFilter, urlState.query, pagination]);

  // Callback for searchbar, called whenever user updates search
  const setSearch = useCallback(
    (query) => {
      setUrlState({ query: serialize(query) });
    },
    [setUrlState]
  );

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
  };

  const smUp = useMediaQuery((theme) => theme.breakpoints.up("sm"));
  const smDown = useMediaQuery((theme) => theme.breakpoints.down("sm"));

  let content = (
    <SearchBar
      search={setSearch}
      query={deserialize(urlState.query)}
      loading={loading}
      placeholder="Search in Record"
    >
      {smDown ? <RecordList records={records} /> : null}
      {smUp ? (
        <RecordTable
          records={records}
          loading={loading}
          pagination={pagination}
          onPage={onPage}
        />
      ) : null}
    </SearchBar>
  );

  return (
    <RootPaper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justifyContent="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={deserialize(urlState.tab)}
              onChange={handleTabChange}
            >
              <Tab label={<Typography variant="h5">All</Typography>} />
              <Tab label={<Typography variant="h5">Only active</Typography>} />
              <Tab
                label={<Typography variant="h5">Only inactive</Typography>}
              />
            </Tabs>
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </RootPaper>
  );
}
